from mixcoatl.resource import Resource
from mixcoatl.decorators.lazy import lazy_property

class Cloud(Resource):
    path = 'geography/Cloud'
    collection_name = 'clouds'
    primary_key = 'cloud_id'

    def __init__(self, cloud_id = None, *args, **kwargs):
        Resource.__init__(self)
        self.__cloud_id = cloud_id

    @property
    def cloud_id(self):
        return self.__cloud_id

    @lazy_property
    def cloud_provider_console_url(self):
        return self.__cloud_provider_console_url

    @lazy_property
    def cloud_provider_logo_url(self):
        return self.__cloud_provider_logo_url

    @lazy_property
    def cloud_provider_name(self):
        return self.__cloud_provider_name

    @lazy_property
    def compute_account_number_label(self):
        return self.__compute_account_number_label

    @lazy_property
    def compute_access_key_label(self):
        return self.__compute_access_key_label

    @lazy_property
    def compute_x509_cert_label(self):
        return self.__compute_x509_cert_label

    @lazy_property
    def compute_x509_key_label(self):
        return self.__compute_x509_key_label

    @lazy_property
    def compute_delegate(self):
        return self.__compute_delegate

    @lazy_property
    def compute_endpoint(self):
        return self.__compute_endpoint

    @lazy_property
    def compute_secret_key_label(self):
        return self.__compute_secret_key_label

    @lazy_property
    def documentation_label(self):
        return self.__documentation_label

    @lazy_property
    def name(self):
        return self.__name

    @lazy_property
    def private_cloud(self):
        return self.__private_cloud

    @lazy_property
    def status(self):
        return self.__status

    @classmethod
    def all(cls):
        r = Resource(cls.path)
        c = r.get()
        if r.last_error is None:
            return [cls(i['cloudId']) for i in c[cls.collection_name]]
        else:
            return r.last_error


from mixcoatl.resource import Resource
from mixcoatl.decorators.lazy import lazy_property

class Group(Resource):
    path = 'admin/Group'
    collection_name = 'groups'
    primary_key = 'group_id'

    def __init__(self, group_id=None, *args, **kwargs):
        Resource.__init__(self)
        self.__group_id = group_id

    @property
    def group_id(self):
        return self.__group_id

    @lazy_property
    def description(self):
        return self.__description

    @lazy_property
    def name(self):
        return self.__name

    @lazy_property
    def status(self):
        return self.__status

    @lazy_property
    def customer(self):
        return self.__customer

    @classmethod
    def all(cls, **kwargs):
        r = Resource(cls.path)
        if 'details' in kwargs:
            r.request_details = kwargs['details']
        else:
            r.request_details = 'basic'

        x = r.get()
        if r.last_error is None:
            return [cls(i['groupId']) for i in x[cls.collection_name]]
        else:
            return x.last_error


import json

from mixcoatl.resource import Resource
from mixcoatl.decorators.lazy import lazy_property
from mixcoatl.decorators.validations import required_attrs

class ApiKey(Resource):
    """An API key is an access key and secret key that provide API access into enStratus."""

    path = 'admin/ApiKey'
    collection_name = 'apiKeys'
    primary_key = 'access_key'

    def __init__(self, access_key = None, *args, **kwargs):
        Resource.__init__(self)
        self.__access_key = access_key

    @property
    def access_key(self):
        """The primary identifier of the `ApiKey`. Same as `ES_ACCESS_KEY`"""
        return self.__access_key

    @lazy_property
    def account(self):
        """`dict` - The account with which this API key is associated."""
        return self.__account

    @lazy_property
    def activation(self):
        """`str` - The date and time when this key was activated."""
        return self.__activation

    @lazy_property
    def expiration(self):
        """`str` - The date and time when this API key should automatically be made inactivate."""
        return self.__expiration

    @expiration.setter
    def expiration(self, e):
        self.__expiration = e

    @lazy_property
    def customer(self):
        """`dict` - The customer to whom this API key belongs."""
        return self.__customer

    @lazy_property
    def customer_management_key(self):
        """`bool` - Identifies whether or not this key can be used across all customer accounts."""
        return self.__customer_management_key

    @lazy_property
    def description(self):
        """`str` - A user-friendly description of this API key."""
        return self.__description

    @description.setter
    def description(self, d):
        self.__description = d

    @lazy_property
    def name(self):
        """`str` - The user-friendly name used to identify the key."""
        return self.__name

    @name.setter
    def name(self, n):
        self.__name = n

    @lazy_property
    def secret_key(self):
        """`str` - The secret part of this API key."""
        return self.__secret_key

    @lazy_property
    def state(self):
        """`str` - The status of the key *(i.e. `ACTIVE`)*"""
        return self.__state

    @lazy_property
    def system_management_key(self):
        """`bool` - Identifies if the key can be used for enStratus system management functions"""
        return self.__system_management_key

    @lazy_property
    def user(self):
        """`dict` - The user associated with this API key. Account-level keys return `{'user_id': -1}`"""
        return self.__user

    @required_attrs(['description', 'name'])
    def create(self):
        """Call the API to generate an API key from the current instance of `ApiKey`"""

        payload = {'generateApiKey':[{'description':self.description, 'name':self.name}]}
        s = self.post(data=json.dumps(payload))
        if self.last_error is None:
            self.__access_key = s['apiKeys'][0]['accessKey']
            self.load()
        else:
            raise ApiKeyGenerationException(self.last_error)


    @classmethod
    def generate_api_key(cls, key_name, description, expiration=None):
        """Generates a new API key

        >>> ApiKey.generate_api_key('my-api-key', 'this is my api key')
        {'access_key':'ABCDEFGHIJKL':....}

        :param key_name: the name for the key
        :type key_name: str.
        :param description: the description for the key
        :type description: str.
        :param expiration: *unused for now*
        :type expiration: str.
        :returns: :class:`ApiKey`
        :raises: :class:`ApiKeyGenerationException`
        """
        
        a = cls()
        a.name = key_name
        a.description = description
        a.create()
        return a

    @classmethod
    def all(cls, keys_only=False, **kwargs):
        """Get all api keys

        The keys used to make the original request determine
        the visible results.

        :param keys_only: Only return `access_key` instead of `ApiKey` objects
        :type keys_only: bool.
        :param detail: The level of detail to return - `basic` or `extended`
        :type detail: str.
        :param account_id: Display all system keys belonging to `account_id`
        :type account_id: int.
        :param user_id: Display all keys belonging to `user_id`
        :type user_id: int.
        :returns: `list` - List of `ApiKey` or `list` -- List of `access_key` 
        """

        r = Resource(cls.path)
        if 'detail' in kwargs:
            r.request_details = kwargs['detail']
        else:
            r.request_details = 'basic'
        
        if 'account_id' in kwargs:
            params = {'accountId': kwargs['account_id']}
        elif 'user_id' in kwargs:
            params = {'userId': kwargs['user_id']}
        else:
            params = {}

        c = r.get(params=params)
        if r.last_error is None:
            if keys_only is True:
                return [i['accessKey'] for i in c[cls.collection_name]]
            else:
                return [cls(i['accessKey']) for i in c[cls.collection_name]]
        else:
            raise ApiKeyException(r.last_error['error']['message'])

class ApiKeyException(BaseException): pass
class ApiKeyGenerationException(ApiKeyException): pass


from mixcoatl.settings.load_settings import settings
import mixcoatl.auth as auth
import requests as r
from mixcoatl.decorators.lazy import lazy_property

class Resource(object):
    """The base class for all resources returned from an enStratus API call
    By default all resources are largely represented as a `dict`-alike object
    that mirrors the JSON response from the enStratus API with keys converted
    from camel-case to snake-case.

    For instance:

        * The enStratus API JSON for a resource looks something like this:

            .. code-block:: yaml

                {
                  'someId':1,
                  'name':'my-name',
                  'cloud':{'cloudId':12345}
                }

        * All keys (nested or top-level) will be converted to snake-case:
            - `someId` becomes `some_id`
            - `cloudId` becomes `cloud_id`
        * The top-level keys will be converted to getters (and in some cases - setters) on the Resource.
            - `some_id` will now be the resource's primary identifier: `resource.some_id`
            - `name` will likely be a getter and a setter as `name` is usually a mutable attributes
            - `cloud` will be converted to a setter with a value of `{'cloud_id':123345}`
        * In cases where a setter would actually need to set a nested value, it will do so. However
            the getter will still return the original data structure as appropriate

        .. warning::

            The Resource object, while looking much like a `dict`, is not a `dict` proper.
            If you need an actual `dict`, call `to_dict()` on your instance.
    """
    
    #: The request path of the resource
    path = None
    #: The top-level grouping of a resource in the API response
    collection_name = None
    #: The unique identifier of an individual resource
    primary_key = None

    def __init__(self, base_path=None, request_details = 'extended'):
        if base_path is None:
            try:
                self.__path = self.__class__.path
            except:
                raise AttributeError('you must override base_path')
        else:
            self.__path = base_path
        self.__last_request = None
        self.__last_error = None
        self.__current_job = None
        self.__request_details = request_details
        self.pending_changes = {}

    def __props(self):
        p = [k for k,v in self.__class__.__dict__.items() if type(v) in [lazy_property, property]]
        rp = ['last_error', 'path', 'last_request', 'current_job']
        return p + rp


    def __repr__(self):
        from mixcoatl.utils import convert
        d = {}
        for x in self.__props():
            try:
                if x == 'last_request':
                    d[x] = str(getattr(self, x))
                else:
                    d[x] = getattr(self, x)
            except AttributeError:
                d[x] = None
            except KeyError:
                d[x] = None
        return repr(convert(d))

    @property
    def request_details(self):
        """The level of detail used in the API call: `basic` or `extended`"""
        return self.__request_details

    @request_details.setter
    def request_details(self, level):
        self.__request_details = level

    @property
    def path(self):
        """The path used in the API request (e.g. ``admin/Job``)"""
        return self.__path

    @path.setter
    def path(self, p):
        self.__path = p

    @property
    def last_request(self):
        """The :class:`Request` object of the most recent API call"""
        return self.__last_request

    @last_request.setter
    def last_request(self, lr):
        self.__last_request = lr

    @property
    def last_error(self):
        """The last error message, if any, returned from the most recent API call"""
        return self.__last_error

    @last_error.setter
    def last_error(self, le):
        self.__last_error = le

    @property
    def current_job(self):
        """The current :class:`Job`, if any, of an asynchronous API call"""
        return self.__current_job

    @current_job.setter
    def current_job(self, cj):
        self.__current_job = cj

    def load(self):
        """(Re)load the current object's attributes from an API call"""
        from mixcoatl.utils import uncamel_keys
        reserved_words = ['type']
        p = self.path+"/"+str(getattr(self, self.__class__.primary_key))

        #self.request_details = 'extended'
        s = self.get(p)
        if self.last_error is None:
            scope = uncamel_keys(s[self.__class__.collection_name][0])
            for k in scope.keys():
                if k in reserved_words:
                    the_key = 'e_'+k
                else:
                    the_key = k
                nk = '_%s__%s' % (self.__class__.__name__, the_key)
                if the_key not in self.__props():
                    raise AttributeError('Key found without accessor: %s' % k)
                else:
                    setattr(self, nk, scope[k])
                    self.loaded = True
        else:
            return self.last_error

    def __doreq(self, method, *args, **kwargs):
        """Performs the actual API call

        * calls `auth.get_sig` for signed headers
        * issues the requested :attr:`method` against the API endpoint
        * Handles requests appropriately based on sync/async nature of the call
            based on enStratus API documentation
        """
        failures = [400, 403, 404, 409, 500, 501, 503]
        sig = auth.get_sig(method, self.path)
        url = settings.endpoint+'/'+self.path

        headers = {'x-esauth-access': sig['access_key'],
        'x-esauth-timestamp': str(sig['timestamp']),
        'x-esauth-signature': str(sig['signature']),
        'x-es-details': self.__request_details,
        'Accept': 'application/json',
        'User-Agent': sig['ua']}

        #results = getattr(r, method.lower())(url, headers=headers, *args, **kwargs)
        results = r.request(method, url, headers=headers, **kwargs)

        self.last_error = None
        self.last_request = results
        if results.status_code in failures:
            self.last_error = results.json()
            return self.last_error

        if method == 'GET':
            try:
                results.raise_for_status()
                self.last_error = None
                return results.json()
            except r.exceptions.HTTPError:
                self.last_error = results.json()
                return False
        if method == 'DELETE':
            if results.status_code !=204:
                self.last_error = results.json()
                return False
            else:
                return True
        if method == 'PUT':
            if results.status_code == 202:
                self.current_job = results.json()['jobs'][0]['jobId']
                return results.json()
            elif results.status_code == 204:
                return True
            else:
                self.last_error = results.json()
                return self.last_error
        if method == 'POST':
            if results.status_code in [201,202]:
                if results.status_code == 202:
                    self.current_job = results.json()['jobs'][0]['jobId']
                return results.json()
            else:
                self.last_error = results.json()
                return False

    def set_path(self, path=None):
        if path is None:
            path = self.path
        else:
            self.path = path

    def get(self, path=None, **kwargs):
        """Perform an HTTP `GET` against the API endpoint for the current resource"""
        self.set_path(path)
        return self.__doreq('GET', **kwargs)

    def post(self, path=None, **kwargs):
        """Perform an HTTP `POST` against the API endpoint for the current resource"""
        self.set_path(path)
        return self.__doreq('POST', **kwargs)

    def put(self, path=None, **kwargs):
        """Perform an HTTP `PUT` against the API endpoint for the current resource"""
        self.set_path(path)
        return self.__doreq('PUT', **kwargs)

    def delete(self, path=None, **kwargs):
        """Perform an HTTP `DELETE` against the API endpoint for the current resource"""
        self.set_path(path)
        return self.__doreq('DELETE', **kwargs)

    def pprint(self):
        """The prettyprint formatted representation of the current resource"""
        import pprint
        pprint.pprint(eval(repr(self)))

    def to_dict(self):
        """The `dict` representation of the current resource"""
        return eval(repr(self))

    def track_change(self, var, new_value):
        prev = getattr(self, var)
        self.pending_changes[var] = {'old': prev, 'new':new_value}



from mixcoatl.resource import Resource
from mixcoatl.decorators.lazy import lazy_property

class ServerProduct(Resource):
    path = 'infrastructure/ServerProduct'
    collection_name = 'serverProducts'
    primary_key = 'product_id'

    def __init__(self, product_id = None, *args, **kwargs):
        Resource.__init__(self)
        self.__product_id = product_id

    @property
    def product_id(self):
        return self.__product_id
        
    @lazy_property
    def cloud(self):
        return self.__cloud

    @lazy_property
    def architecture(self):
        return self.__architecture

    @lazy_property
    def cpu_count(self):
        return self.__cpu_count

    @lazy_property
    def cpu_speed_in_mhz(self):
        return self.__cpu_speed_in_mhz

    @lazy_property
    def currency(self):
        return self.__currency

    @lazy_property
    def description(self):
        return self.__description

    @lazy_property
    def disk_size_in_gb(self):
        return self.__disk_size_in_gb

    @lazy_property
    def hourly_rate(self):
        return self.__hourly_rate

    @lazy_property
    def name(self):
        return self.__name

    @lazy_property
    def platform(self):
        return self.__platform

    @lazy_property
    def provider_product_id(self):
        return self.__provider_product_id

    @lazy_property
    def provider_region_id(self):
        return self.__provider_region_id

    @lazy_property
    def ram_in_mb(self):
        return self.__ram_in_mb

    @lazy_property
    def software(self):
        return self.__software

    @classmethod
    def all(cls, region_id):
        from mixcoatl.utils import uncamel_keys
        r = Resource(cls.path)
        params = {'regionId':region_id}
        c = r.get(params=params)
        if r.last_error is None:
            return [cls(i['productId']) for i in c[cls.collection_name]]
        else:
            return r.last_error


from mixcoatl.resource import Resource
from mixcoatl.decorators.lazy import lazy_property

class User(Resource):
    path = 'admin/User'
    collection_name = 'users'
    primary_key = 'user_id'

    def __init__(self, user_id=None, *args, **kwargs):
        Resource.__init__(self)
        self.__user_id = user_id

    @property
    def user_id(self):
        return self.__user_id

    @lazy_property
    def alpha_name(self):
        return self.__alpha_name

    @lazy_property
    def customer(self):
        return self.__customer

    @lazy_property
    def editable(self):
        return self.__editable

    @lazy_property
    def email(self):
        return self.__email

    @lazy_property
    def family_name(self):
        return self.__family_name

    @lazy_property
    def given_name(self):
        return self.__given_name

    @lazy_property
    def groups(self):
        return self.__groups

    @lazy_property
    def has_cloud_api_access(self):
        return self.__has_cloud_api_access

    @lazy_property
    def has_cloud_console_access(self):
        return self.__has_cloud_console_access

    @lazy_property
    def status(self):
        return self.__status

    @lazy_property
    def time_zone(self):
        return self.__time_zone

    @lazy_property
    def vm_login_id(self):
        return self.__vm_login_id

    @lazy_property
    def ssh_public_key(self):
        return self.__ssh_public_key

    @classmethod
    def all(cls, **kwargs):
        r = Resource(cls.path)
        if 'details' in kwargs:
            r.request_details = kwargs['details']
        else:
            r.request_details = 'basic'

        x = r.get()
        if r.last_error is None:
            return [cls(i['userId']) for i in x[cls.collection_name]]
        else:
            return x.last_error


from mixcoatl.resource import Resource
from mixcoatl.decorators.lazy import lazy_property

class FirewallRule(Resource):
    path = 'network/FirewallRule'
    collection_name = 'rules'
    primary_key = 'firewall_rule_id'

    def __init__(self, firewall_rule_id = None, *args, **kwargs):
        Resource.__init__(self)
        self.__firewall_rule_id = firewall_rule_id

    @property
    def firewall_rule_id(self):
        return self.__firewall_rule_id
    
    @lazy_property
    def direction(self):
        return self.__direction

    @lazy_property
    def firewall(self):
        return self.__firewall

    @lazy_property
    def network_address(self):
        return self.__network_address

    @lazy_property
    def protocol(self):
        return self.__protocol

    @classmethod
    def all(cls, firewall_id):
        from mixcoatl.utils import uncamel_keys
        r = Resource(cls.path)
        r.request_details = 'extended'
        params = {'firewallId':firewall_id}
        c = r.get(params=params)
        if r.last_error is None:
            rules = [i['firewallRuleId'] for i in c[cls.collection_name]]
            return [cls(rule) for rule in rules]
        else:
            return r.last_error


class LazyPropertyException(BaseException): pass

class lazy_property(object):
    """decorator for marking an attribute as lazy loaded

    When using the `@lazy_property` decorator, the subsequent function is
    in essence made a property that is lazily loaded.

    In practice, all attributes of a given resource should be lazy except
    the resource's id. This ensures that simply instantiating a resource will
    not cause an expensive API call to be made.

    .. note::

        * lazy properties adhere to the descriptor protocol
        * Classes with lazy properties will need to implement a `load()` function
        * Once any lazy property is loaded, **ALL** lazy properties in the instance are loaded as a result of the API call.
        * Once all lazy properties are loaded, no further API calls will be made for the current instance.

    :raises: :class:`LazyPropertyException`
    """
    def __init__(self, func=None):

        self._func = func
        self.__doc__ = func.__doc__
        self.__name__ = func.__name__
        self._sfunc = None

    def __get__(self, instance, owner=None):
        myname = self.__name__

        if instance is None: return self

        if self._func is None:
            raise AttributeError, "unknown attribute %s" % myname
        # Check if we've already loaded from API
        if 'loaded' in instance.__dict__:
            if myname in dir(instance):
                return self._func(instance)
            else:
                raise AttributeError, "unknown attribute %s" % myname
        elif myname in instance.__dict__:
            return self._func(instance)
        else:
            if getattr(instance, instance.primary_key) is not None:
                try:
                    instance.load()
                except AttributeError as detail:
                    if instance.last_error is not None:
                        return instance.last_error
                    else:
                        raise LazyPropertyException(detail)

        return self._func(instance)

    def __set__(self, instance, value):
        if self._sfunc is None:
            raise TypeError, "immutable attribute: %s" % self.__name__
        else:
            self._sfunc(instance, value)

    def setter(self, sfunc):
        self._sfunc = sfunc
        return self



import os

from mixcoatl.exceptions import ConfigException

class Config(object):
    # pylint: disable-msg=E0710
    def __init__(self):
        self.access_key = None
        self.secret_key = None
        self.endpoint = None
        self.user_agent = 'mixcoatl'
        self.api_version = None
        self.basepath = None

    def configure(self):
        if self.access_key is None:
            if 'ES_ACCESS_KEY' in os.environ:
                self.set_access_key(os.environ['ES_ACCESS_KEY'])
            else:
                raise ConfigException('missing ES_ACCESS_KEY')
        if self.secret_key is None:
            if 'ES_SECRET_KEY' in os.environ:
                self.set_secret_key(os.environ['ES_SECRET_KEY'])
            else:
                raise ConfigException('missing ES_SECRET_KEY')
        if self.api_version is None:
            if 'ES_API_VERSION' in os.environ:
                self.set_api_version(os.environ['ES_API_VERSION'])
            else:
                self.set_api_version('2012-06-15')
            self.set_basepath('/api/enstratus/%s' % self.api_version)

        if self.endpoint is None:
            if 'ES_ENDPOINT' in os.environ:
                self.set_endpoint(os.environ['ES_ENDPOINT'])
            else:
                self.set_endpoint('https://api.enstratus.com'+self.basepath)

    def set_access_key(self, key):
        self.access_key = key

    def set_secret_key(self, key):
        self.secret_key = key

    def set_endpoint(self, ep):
        self.endpoint = ep

    def set_api_version(self, version):
        self.api_version = version

    def set_basepath(self, basepath):
        self.basepath = basepath



from mixcoatl.resource import Resource
from mixcoatl.decorators.lazy import lazy_property

class Region(Resource):
    path = 'geography/Region'
    collection_name = 'regions'
    primary_key = 'region_id'

    def __init__(self, region_id = None, *args, **kwargs):
        Resource.__init__(self)
        self.__region_id = region_id

    @property
    def region_id(self):
        return self.__region_id

    @lazy_property
    def cloud(self):
        return self.__cloud

    @lazy_property
    def customer(self):
        return self.__customer

    @lazy_property
    def jurisdiction(self):
        return self.__jurisdiction

    @lazy_property
    def name(self):
        return self.__name

    @lazy_property
    def provider_id(self):
        return self.__provider_id

    @lazy_property
    def status(self):
        return self.__status

    @lazy_property
    def description(self):
        return self.__description

    @classmethod
    def all(cls,deref=False):
        r = Resource(cls.path)
        c = r.get()
        if r.last_error is None:
            return [cls(item['regionId']) for item in c[cls.collection_name]]
        else:
            return r.last_error


import json

from mixcoatl.resource import Resource
from mixcoatl.decorators.lazy import lazy_property
from mixcoatl.utils import camelize

class Account(Resource):
    """An account object represents an enStratus account held by an enStratus customer."""

    path = 'admin/Account'
    collection_name = 'accounts'
    primary_key = 'account_id'

    def __init__(self, account_id = None, *args, **kwargs):
        Resource.__init__(self)
        self.__account_id = account_id

    @property
    def account_id(self):
        """`int` - The unique ID of this account"""
        return self.__account_id

    @lazy_property
    def alert_configuration(self):
        """`dict` - The configuration of alert preferences for this account."""
        return self.__alert_configuration

    @lazy_property
    def billing_system_id(self):
        """`int` - The ID associated with this account that may appear on invoices."""
        return self.__billing_system_id

    @lazy_property
    def cloud_subscription(self):
        """`dict` or `None` -- Information about the cloud for this account"""
        return self.__cloud_subscription

    @lazy_property
    def configured(self):
        """`bool` - Has this account has been tied to an account with a cloud provider"""
        return self.__configured

    @lazy_property
    def customer(self):
        """`dict` - The enStratus customer record to which this account belongs"""
        return self.__customer

    @lazy_property
    def default_budget(self):
        """The unique id of the billing code that is the default for discovered resources"""
        return self.__default_budget

    @lazy_property
    def name(self):
        """`str` - User-friendly name used to identify the account"""
        return self.__name

    @lazy_property
    def owner(self):
        """`dict` - user who owns this account"""
        return self.__owner

    @lazy_property
    def plan_id(self):
        """`int` - pricing plan associated with this account"""
        return self.__plan_id

    @lazy_property
    def provisioned(self):
        """`bool` - Is this account in goodstanding and managed by enStratus"""
        return self.__provisioned

    @lazy_property
    def status(self):
        """`str` - The current account payment status"""
        return self.__status

    @lazy_property
    def subscribed(self):
        """`bool` - If the account is configured and the cloud account is working with enStratus"""
        return self.__subscribed


    @classmethod
    def all(cls, keys_only = False, **kwargs):
        """Get all accounts

        >>> Account.all(detail='basic')
        [{'account_id':12345,....}]

        >>> Account.all(keys_only=True)
        [12345]

        :param keys_only: Only return `account_id` instead of `Account` objects
        :type keys_only: bool.
        :param detail: The level of detail to return - `basic` or `extended`
        :type detail: str.
        :param cloud_id: Only show accounts tied to the given cloud
        :type cloud_id: int.
        :returns: `list` - A list of `Account` or `list` -- A list of `account_id`
        :raises: :class:`AccountException`
        """

        r = Resource(cls.path)
        if 'detail' in kwargs:
            r.request_details = kwargs['detail']
        else:
            r.request_details = 'basic'

        if 'cloud_id' in kwargs:
            params = {'cloudId': kwargs['cloud_id']}
        else:
            params = {}

        c = r.get(params=params)
        if r.last_error is None:
            if keys_only is True:
                return [i[camelize(cls.primary_key)] for i in c[cls.collection_name]]
            else:
                return [cls(i[camelize(cls.primary_key)]) for i in c[cls.collection_name]]
        else:
            raise AccountException(r.last_error['error']['message'])

class AccountException(BaseException): pass



from mixcoatl.resource import Resource
from mixcoatl.decorators.lazy import lazy_property

class DataCenter(Resource):
    path = 'geography/DataCenter'
    collection_name = 'dataCenters'
    primary_key = 'data_center_id'

    def __init__(self, data_center_id = None, *args, **kwargs):
        Resource.__init__(self)
        self.__data_center_id = data_center_id

    @property
    def data_center_id(self):
        return self.__data_center_id

    @lazy_property
    def description(self):
        return self.__description

    @lazy_property
    def name(self):
        return self.__name

    @lazy_property
    def provider_id(self):
        return self.__provider_id

    @lazy_property
    def region(self):
        return self.__region

    @lazy_property
    def status(self):
        return self.__status

    @classmethod
    def all(cls,region_id):
        r = Resource(cls.path)
        params = {'regionId':region_id}
        c = r.get(params=params)
        if r.last_error is None:
            x = [cls(i['dataCenterId']) for i in c[cls.collection_name]]
            return x
        else:
            return r.last_error


from mixcoatl.resource import Resource
from mixcoatl.decorators.lazy import lazy_property
from mixcoatl.utils import camelize

class LoadBalancer(Resource):
    path = 'network/LoadBalancer'
    collection_name = 'loadBalancers'
    primary_key = 'load_balancer_id'

    def __init__(self, load_balancer_id=None, *args, **kwargs):
        Resource.__init__(self)
        self.__load_balancer_id = load_balancer_id

    @property
    def load_balancer_id(self):
        return self.__load_balancer_id

    @lazy_property
    def customer(self):
        return self.__customer

    @lazy_property
    def status(self):
        return self.__status

    @lazy_property
    def provider_id(self):
        return self.__provider_id

    @lazy_property
    def description(self):
        return self.__description

    @lazy_property
    def region(self):
        return self.__region

    @lazy_property
    def budget(self):
        return self.__budget

    @lazy_property
    def owning_groups(self):
        return self.__owning_groups

    @lazy_property
    def cname_based(self):
        return self.__cname_based

    @lazy_property
    def address(self):
        return self.__address

    @lazy_property
    def owning_account(self):
        return self.__owning_account

    @lazy_property
    def owning_user(self):
        return self.__owning_user

    @lazy_property
    def cloud(self):
        return self.__cloud

    @lazy_property
    def name(self):
        return self.__name

    @lazy_property
    def data_centers(self):
        return self.__data_centers

    @lazy_property
    def servers(self):
        return self.__servers

    @lazy_property
    def listeners(self):
        return self.__listeners


    @classmethod
    def all(cls, **kwargs):
        r = Resource(cls.path)
        if 'details' in kwargs:
            r.request_details = kwargs['details']
        else:
            r.request_details = 'basic'

        x = r.get()
        if r.last_error is None:
            return [cls(i[camelize(cls.primary_key)]) for i in x[cls.collection_name]]
        else:
            return x.last_error


from mixcoatl.resource import Resource
from mixcoatl.decorators.lazy import lazy_property

class BillingCode(Resource):

    path = 'admin/BillingCode'
    collection_name = 'billingCodes'
    primary_key = 'billing_code_id'

    def __init__(self, billing_code_id = None, *args, **kwargs):
        Resource.__init__(self)
        self.__billing_code_id = billing_code_id

    @property
    def billing_code_id(self):
        return self.__billing_code_id

    @lazy_property
    def budget_state(self):
        return self.__budget_state
    
    @lazy_property
    def current_usage(self):
        return self.__current_usage
    
    @lazy_property
    def customer(self):
        return self.__customer
    
    @lazy_property
    def description(self):
        return self.__description
    
    @lazy_property
    def finance_code(self):
        return self.__finance_code
    
    @lazy_property
    def name(self):
        return self.__name 
    
    @lazy_property
    def projected_usage(self):
        return self.__projected_usage
    
    @lazy_property
    def status(self):
        return self.__status

    @classmethod
    def all(cls, **kwargs):
        r = Resource(cls.path)
        if 'details' in kwargs:
            r.request_details = kwargs['details']
        else:
            r.request_details = 'basic'

        x = r.get()
        if r.last_error is None:
            return [cls(i['billingCodeId']) for i in x[cls.collection_name]]
        else:
            return x.last_error


from mixcoatl.resource import Resource
from mixcoatl.decorators.lazy import lazy_property
from mixcoatl.utils import camelize

import time

class Job(Resource):
    path = 'admin/Job'
    collection_name = 'jobs'
    primary_key = 'job_id'

    def __init__(self, job_id = None, **kwargs):
        """
        Initialize a new Job object from the provided job_id
        """
        Resource.__init__(self)
        self.__job_id = job_id

    @property
    def job_id(self):
        """Return the job's ID. *Immutable*"""

        return self.__job_id

    @lazy_property
    def status(self):
        """Return the job's status. *Immutable*"""

        return self.__status

    @lazy_property
    def description(self):
        """Return the job's description. *Immutable*"""

        return self.__description

    @lazy_property
    def message(self):
        """Return the job's message. *Immutable*"""

        return self.__message

    @lazy_property
    def start_date(self):
        """Return the job's start date. *Immutable*"""

        return self.__start_date

    @lazy_property
    def end_date(self):
        """Return the job's stop date. *Immutable*"""
        
        return self.__end_date

    @classmethod
    def all(cls):
        """Get all jobs

        :returns: list -- a list of :class:`Job`'s
        """
        r = Resource(cls.path)
        x = r.get()
        if r.last_error is None:
            return [cls(i[camelize(cls.primary_key)]) for i in x[cls.collection_name]]
        else:
            return r.last_error

    @classmethod
    def wait_for(cls, job_id, status='COMPLETE', callback = None):
        """Blocks execution until :attr:`job_id` returns :attr:`status`

        :param job_id: The ID of the job to wait on
        :type job_id: int.
        :param status: The status to expect before continuing
        :type status: str.
        :param callback: Optional callback to be called with the final ``Job`` when ``status`` is reached
        :type callback: func.
        """
        j = Job(job_id)
        initial_status = j.status
        if initial_status == 'ERROR':
            return False
        while j.status != status:
            time.sleep(5)
            j.load()
            if j.status == 'ERROR':
                if callback is not None:
                    callback(j)
            else:
                continue
        if callback is not None:
            callback(j)
        else:
            return True



from mixcoatl.resource import Resource
from mixcoatl.decorators.lazy import lazy_property

class Snapshot(Resource):
    path = 'infrastructure/Snapshot'
    collection_name = 'snapshots'
    primary_key = 'snapshot_id'

    def __init__(self, snapshot_id = None, *args, **kwargs):
        Resource.__init__(self)
        self.__snapshot_id = snapshot_id

    @property
    def snapshot_id(self):
        return self.__snapshot_id

    @lazy_property
    def available(self):
        return self.__available

    @lazy_property
    def budget(self):
        return self.__budget

    @budget.setter
    def budget(self, budget):
        self.__budget == budget

    @lazy_property
    def cloud(self):
        return self.__cloud

    @lazy_property
    def created_timestamp(self):
        return self.__created_timestamp

    @lazy_property
    def customer(self):
        return self.__customer

    @lazy_property
    def description(self):
        return self.__description

    @description.setter
    def description(self, desc):
        self.__description = desc

    @lazy_property
    def encrypted(self):
        return self.__encrypted

    @lazy_property
    def name(self):
        return self.__name

    @name.setter
    def name(self, name):
        self.__name = name

    @lazy_property
    def label(self):
        return self.__label

    @label.setter
    def label(self, label):
        self.__label = label

    @lazy_property
    def owning_account(self):
        return self.__owning_account

    @lazy_property
    def provider_id(self):
        return self.__provider_id

    @lazy_property
    def region(self):
        return self.__region

    @lazy_property
    def removable(self):
        return self.__removable

    @lazy_property
    def sharable(self):
        return self.__sharable

    @lazy_property
    def size_in_gb(self):
        return self.__size_in_gb

    @lazy_property
    def status(self):
        return self.__status

    @lazy_property
    def volume(self):
        return self.__volume

    @classmethod
    def all(cls, region_id=None):
        r = Resource(cls.path)
        if region_id is not None:
            params = {'regionId':region_id}
            c = r.get(params=params)
        else:
            c = r.get()
        if r.last_error is None:
            return [cls(i['snapshotId']) for i in c[cls.collection_name]]
        else:
            return r.last_error


from mixcoatl.resource import Resource
from mixcoatl.decorators.lazy import lazy_property

# TODO: certain images cause weird redirect
# m = MachineImage(284555) redirect loop
# m = MachineImage(284831) no loop
class MachineImage(Resource):
    path = 'infrastructure/MachineImage'
    collection_name = 'images'
    primary_key = 'machine_image_id'

    def __init__(self, machine_image_id = None, *args, **kwargs):
        Resource.__init__(self, request_details='basic')
        self.__machine_image_id = machine_image_id

    @property
    def machine_image_id(self):
        return self.__machine_image_id
    
    @lazy_property
    def architecture(self):
        return self.__architecture

    @lazy_property
    def cloud(self):
        return self.__cloud

    @lazy_property
    def creation_timestamp(self):
        return self.__creation_timestamp

    @lazy_property
    def customer(self):
        return self.__customer

    @lazy_property
    def budget(self):
        return self.__budget

    @lazy_property
    def description(self):
        return self.__description

    @lazy_property
    def name(self):
        return self.__name

    @lazy_property
    def owning_account(self):
        return self.__owning_account

    @lazy_property
    def owning_cloud_account_number(self):
        return self.__owning_cloud_account_number

    @lazy_property
    def owning_user(self):
        return self.__owning_user

    @lazy_property
    def platform(self):
        return self.__platform

    @lazy_property
    def provider_id(self):
        return self.__provider_id

    @lazy_property
    def region(self):
        return self.__region

    @lazy_property
    def removable(self):
        return self.__removable

    @lazy_property
    def sharable(self):
        return self.__sharable

    @lazy_property
    def status(self):
        return self.__status

    @lazy_property
    def label(self):
        return self.__label

    @lazy_property
    def products(self):
        return self.__products

    @lazy_property
    def agent_version(self):
        return self.__agent_version

    @classmethod
    def all(cls, region_id):
        from mixcoatl.utils import uncamel_keys
        r = Resource(cls.path, request_details='basic')
        params = {'regionId':region_id}
        c = r.get(params=params)
        if r.last_error is None:
            #images = [i['machineImageId'] for i in c[cls.collection_name]]
            #return images
            return [cls(i['machineImageId']) for i in c[cls.collection_name]]
            #return uncamel_keys(c)
        else:
            return r.last_error


#!/usr/bin/env python

import json
import sys
import pprint
sys.path.append('../mixcoatl')
from mixcoatl import resource

# Usage examples (uses normal ES_* env vars for mixcoatl):
# --------------------------------------------------------
# bin/dump.py infrastructure/Server
# bin/dump.py infrastructure/Server "{}" basic
# bin/dump.py infrastructure/Server "{'regionId':12345}"
# bin/dump.py infrastructure/Server/12345 "{}" basic

# Yes I should have used optparse....
if len(sys.argv) < 2:
    sys.exit('Usage: %s basepath <optional query params dict> [basic|extended]' % sys.argv[0])


basepath = sys.argv[1]
if len(sys.argv) == 3:
    params = eval(sys.argv[2])
else:
    params = None

if len(sys.argv) == 4:
    params = eval(sys.argv[2])
    details = sys.argv[3]
else:
    details = 'extended'

r = resource.Resource(basepath)
r.request_details = details
if params is None:
    z = r.get()
else:
    z = r.get(params=params)

output = sys.stdout
headers = r.last_request.headers
data = json.loads(r.last_request.content)
#print('HEADERS\n-------\n')
#pprint.pprint(headers)
#print('\nRESPONSE\n--------\n')
json.dump(data, output, sort_keys=True, indent=2)
output.write('\n')




from mixcoatl.resource import Resource
from mixcoatl.decorators.lazy import lazy_property

class Firewall(Resource):
    path = 'network/Firewall'
    collection_name = 'firewalls'
    primary_key = "firewall_id"

    def __init__(self, firewall_id = None, *args, **kwargs):
        Resource.__init__(self)
        self.__firewall_id = firewall_id

    @property
    def firewall_id(self):
        return self.__firewall_id

    @lazy_property
    def status(self):
        return self.__status

    @lazy_property
    def label(self):
        return self.__label

    @lazy_property
    def owning_account(self):
        return self.__owning_account

    @lazy_property
    def name(self):
        return self.__name

    @lazy_property
    def owning_user(self):
        return self.__owning_user

    @lazy_property
    def description(self):
        return self.__description

    @lazy_property
    def cloud(self):
        return self.__cloud

    @lazy_property
    def provider_id(self):
        return self.__provider_id

    @lazy_property
    def region(self):
        return self.__region

    @lazy_property
    def removable(self):
        return self.__removable

    @lazy_property
    def budget(self):
        return self.__budget

    @lazy_property
    def customer(self):
        return self.__customer

    @property
    def rules(self):
        try:
            return self.__rules
        except AttributeError:
            from mixcoatl.network.firewall_rule import FirewallRule
            rls = FirewallRule.all(self.__firewall_id)
            if len(rls) < 1:
                self.__rules = []
            else:
                self.__rules = rls
            return rls

    @classmethod
    def all(cls, region_id):
        from mixcoatl.utils import uncamel_keys
        r = Resource(cls.path)
        r.request_details = 'extended'
        params = {'regionId':region_id}
        c = r.get(params=params)
        if r.last_error is None:
            return [cls(i['firewallId']) for i in c[cls.collection_name]]
            #return uncamel_keys(c)
        else:
            return r.last_error
